@@ -3,8 +3,7 @@ import pkg_resources
 try:
     version = pkg_resources.require("sequana")[0].version
 except:
-    version = ">=0.7.1"
-
+    version = ">=0.8.3"
 
 try:
     from easydev.logging_tools import Logging
@@ -14,7 +13,6 @@ except:
     logger = colorlog.getLogger("sequana")
 
 
-
 from easydev import CustomConfig
 configuration = CustomConfig("sequana", verbose=False)
 sequana_config_path = configuration.user_config_dir
@@ -25,6 +23,7 @@ from .datatools import sequana_data
 from .assembly import *
 from .adapters import AdapterReader, FindAdaptersFromDesign, Adapter
 from .bamtools import BAM, SAMFlags, SAM, CRAM
+from .bed import BED
 from .bedtools import GenomeCov
 from .cigar import Cigar
 from .coverage import Coverage
@@ -36,9 +35,10 @@ from .freebayes_bcf_filter import BCF_freebayes
 from .itol import ITOL
 from .kraken_builder import KrakenBuilder
 from .krona import KronaMerger
-from .kraken import KrakenResults, KrakenPipeline, KrakenAnalysis, KrakenDownload
+from .kraken import KrakenResults, KrakenPipeline, KrakenAnalysis, KrakenDownload, KrakenSequential
 from .pacbio import PacbioSubreads
 from .phred import Quality
+from .rnadiff import RNADiffResults, plot_venn
 from .running_median import RunningMedian
 from .snaketools import *
 from .snpeff import SnpEff
