@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+
 #
 #  This file is part of Sequana software
 #
@@ -145,8 +145,8 @@ def plot_stats(inputdir=".", outputdir=".",
                filename="snakemake_stats.png", N=1):
     logger.info("Workflow finished. Creating stats image")
     try:
-        SnakeMakeStats("%s/stats.txt" % inputdir, N=N).plot_and_save(
-            outputdir=outputdir, filename=filename)
+        sms = SnakeMakeStats("%s/stats.txt" % inputdir, N=N)
+        sms.plot_and_save(outputdir=outputdir, filename=filename)
     except Exception as err:
         logger.error(err)
         logger.error("Could not process %s/stats.txt file" % inputdir)
@@ -361,7 +361,7 @@ class Module(object):
         self._mf.isvalid(name)
 
         if name not in self._mf.names:
-            raise ValueError("""Sequana error: unknown rule or pipeline {}. 
+            raise ValueError("""Sequana error: unknown rule or pipeline '{}'. 
 Check the source code at:
 
     https://github.com/sequana/sequana/tree/develop/sequana/pipelines and
@@ -557,13 +557,17 @@ or open a Python shell and type::
 
         for req in reqlist:
             # It is either a Python package or an executable
+            if req.startswith("#"):
+                continue
             try:
                 easydev.shellcmd("which %s" % req)
                 logger.debug("Found %s executable" % req)
             except:
                 # is this a Python code ?
                 if len(easydev.get_dependencies(req)) == 0:
-                    logger.error("%s not found !! You must install it" % req)
+                    logger.error("%s not found !! " % req + 
+                        "Let us move on but you must probably install " + 
+                        "it to benefit from all fonctionalities")
                     executable = False
                     missing.append(req)
                 else:
@@ -577,7 +581,7 @@ or open a Python shell and type::
             txt = "Some executable or Python packages are not available:\n"
             txt += "Some functionalities may not work "
             if mode == "warning":
-                print(txt)
+                logger.warning(txt)
             elif mode == "error":
                 txt += "Use \n conda install missing_package_name;"
                 for this in missing:
@@ -835,31 +839,17 @@ class SequanaConfig(object):
             print(err)
             return False
 
-"""
-class DummyManager(object):
-    def __init__(self, filenames=None, samplename="custom"):
-        self.config = {}
-        if isinstance(filenames, list) and len(filenames) == 2:
-            self.paired = True
-            self.samples = {samplename: filenames}
-        elif isinstance(filenames, list) and len(filenames) == 1:
-            self.paired = False
-            self.samples = {samplename: filenames}
-        elif isinstance(filenames, str):
-            self.samples = {samplename: [filenames]}
-            self.paired = False
-"""
 
-class PipelineManagerGeneric(object):
+class PipelineManagerBase(object):
     """
 
     For all files except FastQ, please use this class instead of
     PipelineManager.
 
     """
-    def __init__(self, name, config, sample_func=None):
-
+    def __init__(self, name, config):
         # Make sure the config is valid
+        self.name = name
         cfg = SequanaConfig(config)
 
         # Used by the dynamic rules to defined the location where to copy
@@ -871,6 +861,127 @@ class PipelineManagerGeneric(object):
         self.config.sequana_version = sequana_version
         self.config.pipeline_name = name
 
+    def error(self, msg):
+        msg += ("\nPlease check the content of your config file. You must have "
+                "input_directory set, or input_pattern.")
+        raise SequanaException(msg)
+
+    def getname(self, rulename, suffix=None):
+        """In the basename, include rulename and suffix"""
+        if suffix is None:
+            suffix = ""
+        return self.basename % rulename + suffix
+
+    def getreportdir(self, acronym):
+        """Create the report directory.
+        """
+        return "{1}{0}report_{2}_{1}{0}".format(os.sep, self.sample, acronym)
+
+    def getwkdir(self, rulename):
+        return self.sample + os.sep + rulename + os.sep
+
+    def getlogdir(self, rulename):
+        """ Create log directory: ``*/sample/logs/sample_rule.logs``
+        """
+        return "{1}{0}logs{0}{1}.{2}.log".format(os.sep, self.sample, rulename)
+
+    def getrawdata(self):
+        """Return list of raw data
+
+        If :attr:`mode` is *nowc*, a list of files is returned (one or two files)
+        otherwise, a function compatible with snakemake is returned. This function
+        contains a wildcard to each of the samples found by the manager.
+        """
+        if self.samples is None:
+            raise ValueError("Define the samples attribute as a dictionary with"
+                "sample names as keys and the corresponding location as values.")
+        return lambda wildcards: self.samples[wildcards.sample]
+
+    def plot_stats(self, outputdir=".sequana",
+                 filename="snakemake_stats.png", N=1):
+        logger.info("Creating stats image")
+        try:
+            from sequana.snaketools import SnakeMakeStats
+            sms = SnakeMakeStats("stats.txt", N=N)
+            if sms._parse_data() == {'total_runtime': 0, 
+                'rules': {}, 'files':{}}:
+                return 
+
+            sms.plot_and_save(outputdir=outputdir, filename=filename)
+        except Exception as err:
+            logger.error(err)
+            logger.error("Could not process stats.txt file" )
+
+    def message(self, msg):
+        message(msg)
+
+    def setup(self,  namespace, mode="error", matplotlib="Agg"):
+        """
+
+        90% of the errors come from the fact that users did not set a matplotlib
+        backend properly. In the setup() function, we set the backend to Agg on
+        purpose. One can set this parameter to None to avoid this behaviour
+        """
+        if "__snakefile__" in namespace.keys():
+            self._snakefile = namespace["__snakefile__"]
+        else:
+            filename = self.name + ".rules"
+            # This contains the full path of the snakefile
+            namespace['__snakefile__'] = filename
+            self._snakefile = namespace["workflow"].included_stack[-1]
+
+            # FIXME this is used only in quality_control when using sambamba rule
+            # to remove files.
+            namespace['__pipeline_name__'] = \
+                os.path.split(filename)[1].replace(".rules", "")
+            namespace['expected_output'] = []
+            namespace['toclean'] = []
+
+        # check requirements
+        Module(self.name).check(mode)
+
+        if matplotlib:
+            import matplotlib
+            matplotlib.use('Agg')
+
+    def _get_snakefile(self):
+        return self._snakefile
+    snakefile = property(_get_snakefile)
+
+    def clean_multiqc(self, filename):
+        with open(filename, "r") as fin:
+            with open(filename + "_tmp_", "w") as fout:
+                line = fin.readline()
+                while line:
+                    if """<a href="http://multiqc.info" target="_blank">""" in line:
+                        line = fin.readline() # read the image
+                        line = fin.readline() # read the ending </a> tag
+                    else:
+                        fout.write(line)
+                    line = fin.readline() # read the next line
+        import shutil
+        shutil.move(filename +"_tmp_", filename)
+
+    def teardown(self, extra_dirs_to_remove=[], extra_files_to_remove=[],
+        plot_stats=True):
+
+        # create and save the stats plot
+        if plot_stats:
+            self.plot_stats()
+
+        # add a Makefile
+        cleaner = OnSuccessCleaner(self.name)
+        cleaner.directories_to_remove.extend(extra_dirs_to_remove)
+        cleaner.files_to_remove.extend(extra_files_to_remove)
+        cleaner.add_makefile()
+
+
+
+class PipelineManagerGeneric(PipelineManagerBase):
+    def __init__(self, name, config, sample_func=None):
+        super(PipelineManagerGeneric, self).__init__(name, config)
+
+        cfg = SequanaConfig(config)
         path = cfg.config["input_directory"]
         pattern = cfg.config["input_pattern"]
 
@@ -898,40 +1009,20 @@ class PipelineManagerGeneric(object):
         else:
             self.samples = {str(i+1):filename for i,filename in enumerate(self.ff.realpaths)}
 
-    
-    def getname(self, rulename, suffix=None):
-        """In the basename, include rulename and suffix"""
-        if suffix is None:
-            suffix = ""
-        return self.basename % rulename + suffix
 
-    def getreportdir(self, acronym):
-        """Create the report directory.
-        """
-        return "{1}{0}report_{2}_{1}{0}".format(os.sep, self.sample, acronym)
+class PipelineManagerDirectory(PipelineManagerBase):
+    """
 
-    def getwkdir(self, rulename):
-        return self.sample + os.sep + rulename + os.sep
+    For all files except FastQ, please use this class instead of
+    PipelineManager.
 
-    def getlogdir(self, rulename):
-        """ Create log directory: */sample/logs/sample_rule.logs
-        """
-        return "{1}{0}logs{0}{1}.{2}.log".format(os.sep, self.sample, rulename)
-
-    def getrawdata(self):
-        """Return list of raw data
-
-        If :attr:`mode` is *nowc*, a list of files is returned (one or two files)
-        otherwise, a function compatible with snakemake is returned. This function
-        contains a wildcard to each of the samples found by the manager.
-        """
-        if self.samples is None:
-            raise ValueError("Define the samples attribute as a dictionary with"
-                "sample names as keys and the corresponding location as values.")
-        return lambda wildcards: self.samples[wildcards.sample]
+    """
+    def __init__(self, name, config):
+        super(PipelineManagerDirectory, self).__init__(name, config)
 
 
-class PipelineManager(object):
+
+class PipelineManager(PipelineManagerGeneric):
     """Utility to manage easily the snakemake pipeline
 
     Inside a snakefile, use it as follows::
@@ -977,8 +1068,10 @@ class PipelineManager(object):
         :param pattern: a default pattern if not provided in the configuration
             file as an *input_pattern* field.
         """
+        super(PipelineManagerGeneric, self).__init__(name, config)
+
         cfg = SequanaConfig(config)
-        cfg.config.pipeline_name = name
+        cfg.config.pipeline_name = self.name
         self.pipeline_dir = os.getcwd() + os.sep
 
         # Default mode is the input directory .
@@ -1055,30 +1148,6 @@ class PipelineManager(object):
         self.sample = "{sample}"
         self.basename = "{sample}/%s/{sample}"
 
-    def error(self, msg):
-        msg += ("\nPlease check the content of your config file. You must have "
-                "input_directory set, or input_pattern.")
-        raise SequanaException(msg)
-
-    def getname(self, rulename, suffix=None):
-        """In the basename, include rulename and suffix"""
-        if suffix is None:
-            suffix = ""
-        return self.basename % rulename + suffix
-
-    def getreportdir(self, acronym):
-        """Create the report directory.
-        """
-        return "{1}{0}report_{2}_{1}{0}".format(os.sep, self.sample, acronym)
-
-    def getwkdir(self, rulename):
-        return self.sample + os.sep + rulename + os.sep
-
-    def getlogdir(self, rulename):
-        """ Create log directory: */sample/logs/sample_rule.logs
-        """
-        return "{1}{0}logs{0}{1}.{2}.log".format(os.sep, self.sample, rulename)
-
     def getrawdata(self):
         """Return list of raw data
 
@@ -1088,26 +1157,6 @@ class PipelineManager(object):
         """
         return lambda wildcards: self.samples[wildcards.sample]
 
-    """ probably not required anymore so commented in v0.8.0
-
-    def _get_filenames(self, cfg):
-        filenames = []
-        file1 = cfg.samples.file1
-        file2 = cfg.samples.file2
-        if file1:
-            if os.path.exists(file1):
-                filenames.append(file1)
-            else:
-                raise FileNotFoundError("%s not found" % file1)
-        if file2:
-            if os.path.exists(file2):
-                filenames.append(file2)
-            else:
-                raise FileNotFoundError("%s not found" % file2)
-        return filenames
-    """
-    def message(self, msg):
-        message(msg)
 
 
 def message(mes):
@@ -1165,9 +1214,9 @@ class DOTParser(object):
 
         """
         self.filename = filename
-        self.re_index = re.compile('(\d+)\[')
-        self.re_name = re.compile('label = "(\w+)"')
-        self.re_arrow = re.compile('(\d+) -> (\d+)')
+        self.re_index = re.compile(r'(\d+)\[')
+        self.re_name = re.compile(r'label = "(\w+)"')
+        self.re_arrow = re.compile(r'(\d+) -> (\d+)')
         self.mode = mode
 
     def add_urls(self, output_filename=None, mapper={}, title=None):
@@ -1329,7 +1378,7 @@ class FileFactory(object):
 
         :param pattern: can be a filename, list of filenames, or a global
             pattern (a unix regular expression with wildcards). For instance,
-            "*/*fastq.gz"
+            ``*/*fastq.gz``
 
         .. warning:: Only in Python 3.X supports the recursive global pattern
             for now.
@@ -1443,7 +1492,7 @@ class FastQFactory(FileFactory):
         :param str pattern: a global pattern (e.g., ``H*fastq.gz``)
         :param list extension: not used
         :param str read_tag: regex tag used to join paired end files. Some
-            characters need to be escaped with a '\' to be interpreted as
+            characters need to be escaped with a \ character to be interpreted as
             character. (e.g. '_R[12]_\.fastq\.gz')
         :param bool verbose:
         """
@@ -1492,7 +1541,7 @@ class FastQFactory(FileFactory):
         # If a user uses a . it should be taken into account hence the regex
         # that follows 
         if  self.read_tag:
-            re_read_tag = re.compile(self.read_tag.replace(".", "\."))
+            re_read_tag = re.compile(self.read_tag.replace(r".", r"\."))
             self.tags = list({re_read_tag.split(f)[0] for f in self.basenames})
         else:
             self.tags = list({f.split(".")[0] for f in self.basenames})
@@ -1589,17 +1638,33 @@ class FastQFactory(FileFactory):
         return len(self.tags)
 
 
-def init(filename, namespace):
+"""class Init:
+    class __OnlyOnce:
+        def __init__(self, filename, namespace, mode="error"):
+            init(filename, namespace, mode=mode)
+    instance = None
+    def __init__(self, filename, namespace, mode="error"):
+        if not Init.instance:
+            Init.__OnlyOnce(filename, namespace, mode=mode)
+            instance = True
+        else:
+            pass
+"""
+
+
+def init(filename, namespace, mode="error"):
     """Defines the global variable __snakefile__ inside snakefiles
 
     If not already defined, __snakefile__ is created to hold the name of the
-    pipeline. We also define initialise these variables :
+    pipeline. We also initialise these variables :
 
         * expected_output as an empty list
-        * toclean as an empty  list
+        * toclean as an empty list
 
     Use e.g. in quality_control pipeline for bwa_mem_dynamic option
     """
+    logger.warning("Message for developers. This will be deprecated soon. "
+        "Please use the PipelineManager instead setup() method")
     # Create global name for later
     if "__snakefile__" in namespace.keys():
         pass
@@ -1615,7 +1680,7 @@ def init(filename, namespace):
         namespace['toclean'] = []
 
     # check requirements
-    Module(filename.replace(".rules", "")).check('error')
+    Module(filename.replace(".rules", "")).check(mode)
 
 
 def create_cleanup(targetdir, exclude=['logs']):
@@ -1662,8 +1727,8 @@ def build_dynamic_rule(code, directory):
     fh.close()
     return filename
 
-
-def add_stats_summary_json(json_list, parser):
+"""
+def __add_stats_summary_json(json_list, parser):
     if not parser.stats:
         return
     for jfile in json_list:
@@ -1673,7 +1738,7 @@ def add_stats_summary_json(json_list, parser):
         j = json.dumps(jdict)
         with open(jfile, 'w') as fp:
             print(j, file=fp)
-
+"""
 
 class Makefile(object):
 
@@ -1767,12 +1832,16 @@ print("done")
 
 
 class OnSuccessCleaner(object):
-    """Used in demultiplex pipeline to cleanup final results"""
+    """Used in various sequana pipelines to cleanup final results"""
     def __init__(self, pipeline_name=None, bundle=False):
         self.makefile_filename = "Makefile"
-        self.files_to_remove = ["config.yaml", "multiqc_config.yaml",
+        self.files_to_remove = [
+            "config.yaml",
+            "multiqc_config.yaml",
             "cluster_config.json",
-            "slurm*out", "stats.txt", "schema.yaml"]
+            "slurm*out",
+            "stats.txt",
+            "schema.yaml"]
         if pipeline_name:
             self.files_to_remove.append("{}.sh".format(pipeline_name))
             self.files_to_remove.append("{}.rules".format(pipeline_name))
@@ -1785,7 +1854,7 @@ class OnSuccessCleaner(object):
         self.bundle_output = output
 
     def add_makefile(self):
-        makefile = 'all:\n\techo "sequana cleanup"\n'
+        makefile = 'all:\n\techo "type *make clean* to delete temporary files"\n'
         if self.bundle:
             makefile += "bundle:\n\ttar cvfz {} {}\n".format(
                 self.bundle_output, self.bundle_input)
@@ -1803,7 +1872,7 @@ class OnSuccessCleaner(object):
         with open(self.makefile_filename, "w") as fh:
             fh.write(makefile)
 
-        logger.info("Once done, please clean up the directory using\n'make clean'")
+        logger.info("Once done, please clean up the directory using: 'make clean'")
 
 def get_pipeline_statistics():
     """Get basic statistics about the pipelines
@@ -1841,6 +1910,7 @@ def get_pipeline_statistics():
 
 
 def clean_multiqc(filename):
+    print("Deprecated. Use the pipeline manager instead")
     with open(filename, "r") as fin:
         with open(filename + "_tmp_", "w") as fout:
             line = fin.readline()
